-- ====================================================================
-- LEDTutorial.vhd
-- ====================================================================
-- Note:
-- 
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- library xo2;
-- use xo2.components.all;
-----------------------------------------------------------------------
entity LEDTutorial is
   port(
      rstn      : in std_logic; -- reset active low
      clk_25mhz : in std_logic; -- oscillator 25Mhz
      ld_button : in std_logic;
		sr_button : in std_logic;
		sl_button : in std_logic;
		led_out   : out std_logic_vector(3 downto 0)
   );
end LEDTutorial;
-----------------------------------------------------------------------
architecture rtl_LEDTutorial of LEDTutorial is

   signal counter : std_logic_vector(21 downto 0);
	----------------------------------------------------------
	-- counter reducing only to speed up functional simulation
	----------------------------------------------------------
   --signal counter : std_logic_vector(2 downto 0);
	----------------------------------------------------------
	-- counter reducing only to speed up functional simulation
	----------------------------------------------------------

   signal clk_25mhz_enable : std_logic;

   signal led_reg : std_logic_vector(3 downto 0);

begin

   --------------------------------------------------------
   -- Free runnning counter to generate clock enable signal
   --------------------------------------------------------
   COUNTER_PROC: process(rstn, clk_25mhz)
	begin
      if (rstn = '0') then
         counter          <= (others => '1');
			clk_25mhz_enable <= '0';
      elsif (clk_25mhz = '1' and clk_25mhz'event) then
		--elsif (rising_edge(clk_25mhz)) then
			counter <= counter + 1;
			if (counter = 0) then
				clk_25mhz_enable <= '1';
			else
				clk_25mhz_enable <= '0';
			end if;
		end if;
	end process COUNTER_PROC;
   --------------------------------------------------------
   -- Free runnning counter to generate clock enable signal
   --------------------------------------------------------

   --------------------------------------------------
   -- LED process to manage LOAD, SHIFT RIGHT or LEFT
   --------------------------------------------------
   LED_PROC: process(rstn, clk_25mhz)
	begin
      if (rstn = '0') then
         led_reg <= (others => '1');
		elsif (clk_25mhz = '1' and clk_25mhz'event) then
		--elsif (rising_edge(clk_25mhz)) then
			if (clk_25mhz_enable = '1') then
				-- check LOAD, SHIFT RIGHT and SHIFT LEFT buttons
				if (ld_button = '0') then
					led_reg <= "0110";
				elsif (sr_button = '0') then
					led_reg(2 downto 0) <= led_reg(3 downto 1);
					led_reg(3)          <= led_reg(0);
				elsif (sl_button = '0') then
					led_reg(3 downto 1) <= led_reg(2 downto 0);
					led_reg(0)          <= led_reg(3);
				end if;
			end if;
      end if;
   end process LED_PROC;
   --------------------------------------------------
   -- LED process to manage LOAD, SHIFT RIGHT or LEFT
   --------------------------------------------------

   -------------
   -- LED output
   -------------
	led_out <= led_reg;
   -------------
   -- LED output
   -------------

end rtl_LEDTutorial;
-----------------------------------------------------------------------